from gtts import gTTS
from bnsttts import logger

class TTS:
    """ Text to speech conversion"""

    def __init__(self):
        pass

    def speak(self, text: str, save_path:str, lang: str = "bn", slow: bool=False) -> None:
        """Speaks given text in given language

        Args:
            text (str): Text to be spoken
            save_path (str): Path to save the audio file
            lang (str, optional): Language code. Defaults to "en".
            slow (bool, optional): Speak slowly. Defaults to False.
        """
        tts = gTTS(text=text, lang=lang, slow=slow)
        tts.save(save_path)


if __name__ == "__main__":
    tts = TTS()

    logger.info("Converting text to speech")
    tts.speak("আমি বাংলায় কথা বলতে পারি", "data/tts/output.mp3", lang="bn")
    logger.info("Text to speech conversion completed")


import os
import sys
from loguru import logger

# Define version
__version__ = "0.0.1"

# Define log directory and file
log_dir = "logs"
log_filepath = os.path.join(log_dir, "bnsttts_logs.log")
os.makedirs(log_dir, exist_ok=True)

# Remove default handler to avoid duplicate logs
logger.remove()

# Add log handlers
logger.add(
    log_filepath,
    format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> | "
           "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>",
    level="INFO",
)
logger.add(
    sys.stdout,
    format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> | <level>{level: <8}</level> | "
           "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>",
    level="INFO",
)

logger.info("Bangla Speech to Text and Text to Speech")


import nemo.collections.asr as nemo_asr
from typing import List
from bnsttts import logger
import librosa

class STT:
    """Speech to Text class using Nemo ASR model"""
    def __init__(self, ):
        self.model_name = "hishab/titu_stt_bn_fastconformer"
        self.model = nemo_asr.models.ASRModel.from_pretrained(
            model_name = self.model_name
        )

    def preprocess(self, audio_file_path: str) -> str:
        """Preprocesses audio file

        Args:
            audio_file_path (str): Path to the audio file

        Returns:
        str: Path to the preprocessed audio file
        """
        y, sr = librosa.load(audio_file_path)
        # Convert stereo audio to mono by averaging channels if necessary
        if y.ndim == 2:
            y = librosa.to_mono(y)
        return y

    def transcribe(self, audio_file_path: str) -> List[str]:
        """Transcribes audio file to text

        Args:
            audio_file_path (str): Path to the audio file

        Returns:
        List[str]: List of transcribed bangla text
        """
        # Convert stereo audio to mono by averaging channels if necessary
        y = self.preprocess(audio_file_path)
        transcription = self.model.transcribe([y])
        return transcription

if __name__ == "__main__":
    stt = STT()
    transcriptions = stt.transcribe("data/stt/01.wav")
    for transcription in transcriptions:
        logger.info(transcription)


import os
import click
from bnsttts.stt import STT
from bnsttts.tts import TTS
from bnsttts import logger


# Click settings
CONTEXT_SETTINGS = {"help_option_names": ["-h", "--help"]}


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument('content', required=True)
@click.option('--output', '-o', type=str, default='output.mp3', help='Where text to audio file will be stored.')
@click.option(
    '--lang', '-l', 
    type=str,
    default='bn', 
    help='Language to speak in (e.g., en, bn')

@click.option('--tts', '-t', is_flag=True, help='For text to speech.')
@click.option('--stt', '-s', is_flag=True, help='For speech to text.')
@click.option('--slow', is_flag=True, help='For slow speech.')

def main(content, output, lang, tts, stt, slow) -> str:
    """
    A CLI tool to convert text to speech or speech to text.

    Args:
        content (str): audio file path or text
        output (str): Where text to audio file will be stored.
        lang (str): Language to speak in (e.g., en, bn)
        tts (bool): For text to speech.
        stt (bool): For speech to text.
        slow (bool): For slow speech.

    Returns:
        str: Path to the generated audio file or transcribe text.
    """
    if tts:
        if not os.path.exists(os.path.dirname(output)):
            logger.info(f"Creating directory {os.path.dirname(output)}")
        tts = TTS()
        tts.speak(content, output, lang=lang, slow=slow)
        logger.info(f"Audio file saved at {output}")
        
    elif stt:
        if not os.path.exists(content):
            logger.error(f"Audio file not found at {content}")
            return
        # Convert audio file to text
        stt = STT()
        transcriptions = stt.transcribe(content)
        logger.info(f"Transcribed text: {transcriptions[0]}")

if __name__ == "__main__":
    # Ensure Click executes the `main()` function with the CLI arguments
    response = main()